/* mikmod.c -- Mikmod-plugin for musicdaemon player
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <pthread.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <mikmod.h>

#include "../../musicd.h"
#include "mikmod-plugin.h"

input_plugin *get_iplugin_info (void);
static void init(void);
static int is_our_file(char *filename);
static void play(char *filename);
static void stop(void);
static void* update_loop(void*);
static int get_time(void);
static const char * strcasestr(const char *haystack, const char *needle);

input_plugin meta_input_ip =
{
    42,
    NULL,
    NULL,
    "Mikmod input plugin",
    init,
    is_our_file,
    play,
    stop,
    NULL,
    NULL,
    get_time,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL
};

static pthread_t decode_thread;
static pthread_t update_thread;
int going = 0;
static int audio;
static int initialized = 0;
static MODULE *module;

input_plugin *get_iplugin_info()
{
    return(&meta_input_ip);
}

static void init()
{
    audio = 0;
    if(initialized)
	return;
    MikMod_InitThreads();
    MikMod_RegisterAllLoaders();
    initialized=1;
    return;
}

static int is_our_file(char *filename)
{
    if(strcasestr(filename, ".s3m") != NULL)
	return(1);
    if(strcasestr(filename, ".it") != NULL)
	return(1);
    if(strcasestr(filename, ".xm") != NULL)
	return(1);
    if(strcasestr(filename, ".mod") != NULL)
	return(1);
    if(strcasestr(filename, ".669") != NULL)
	return(1);
    if(strcasestr(filename, ".amf") != NULL)
	return(1);
    if(strcasestr(filename, ".dsm") != NULL)
	return(1);
    if(strcasestr(filename, ".m15") != NULL)
	return(1);
    if(strcasestr(filename, ".med") != NULL)
	return(1);
    if(strcasestr(filename, ".mtm") != NULL)
	return(1);
    if(strcasestr(filename, ".stm") != NULL)
	return(1);
    if(strcasestr(filename, ".ult") != NULL)
	return(1);
    if(strcasestr(filename, ".imf") != NULL)
	return(1);
    if(strcasestr(filename, ".gdm") != NULL)
	return(1);
    if(strcasestr(filename, ".stx") != NULL)
	return(1);
    if(strcasestr(filename, ".far") != NULL)
	return(1);
    return(0);
}

static void* update_loop(void *none)
{
    if(none){}
    while(Player_Active())
    {
	MikMod_Update();
	usleep(10000);
    }
    going = 0;
    return(0);
}

static void play(char *filename)
{

    if(!going)
    {
	going = 1;
	audio = 1;


	MikMod_RegisterDriver(&mikmod_drv);
	md_mode |= DMODE_SOFT_MUSIC;
	if (MikMod_Init(""))
	{
	    fprintf(stderr, "Could not initialize sound, reason: %s\n",
		    MikMod_strerror(MikMod_errno));
	    return;
	}          

	module = Player_Load(filename, 64, 0);
	if (module)
	{
	    Player_Start(module);

	} else
	    fprintf(stderr, "Could not load module, reason: %s\n",
		    MikMod_strerror(MikMod_errno));

	pthread_create(&update_thread, NULL, update_loop, NULL);

    }
}

static void stop(void)
{
    if(audio)
    {
	audio = 0;
	going = 0;
	Player_Stop();
	Player_Free(module);
	pthread_join(decode_thread, NULL);
	pthread_join(update_thread, NULL);
	meta_input_ip.close_audio();
/*	MikMod_Exit();*/
    }
}

static int get_time(void)
{
    if(going)
    {
	return(10);
    }
    return(-1);
}

const char * strcasestr(const char *haystack, const char *needle)
{
    const char *index;
    int needle_len = strlen(needle);
    int iters = strlen(haystack) - needle_len + 1;   
    for(index = haystack; iters > 0 ; iters--)       
    {
	if(strncasecmp(index, needle, needle_len) == 0)
	    return(index);
	index++;
    }
    return(NULL);
}
