/* musicd-drv.c -- Music daemon driver for mikmod
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <unistd.h>
#include <mikmod.h>

#include "../../musicd.h"
#include "mikmod-plugin.h"

static char buffer[4096];

static BOOL is_present(void)
{
    return(1);
}

static BOOL drv_init(void)
{
    audio_format format;
    int channels;
    
    if(md_mode & DMODE_16BITS)
	format = FMT_S16_LE;
    else
	format = FMT_S8;
    
    if(md_mode & DMODE_STEREO)
	channels = 2;
    else
	channels = 1;
    
    meta_input_ip.open_audio(format, md_mixfreq, channels);
    
    return(VC_Init());
}

static void drv_update(void)
{
    int length;
    
    length = VC_WriteBytes((SBYTE *) buffer, sizeof(buffer));
    
    while(meta_input_ip.buffer_free() < length && going)
	usleep(10000);
    
    if(going)
	meta_input_ip.write_audio(buffer, length);
}

static BOOL drv_reset(void)
{
    VC_Exit();
    return(VC_Init());
}

static void drv_exit(void)
{
    VC_Exit();
}

MDRIVER mikmod_drv =
{
    NULL,
    "musicd",
    "Music daemon's output driver",
    0, 255,

#if (LIBMIKMOD_VERSION > 0x030106)

    "xmms",
    NULL,

#endif

    is_present,
    VC_SampleLoad,
    VC_SampleUnload,
    VC_SampleSpace,
    VC_SampleLength,
    drv_init,
    drv_exit,
    drv_reset,
    VC_SetNumVoices,
    VC_PlayStart,
    VC_PlayStop,
    drv_update,
    NULL,
    VC_VoiceSetVolume,
    VC_VoiceGetVolume,
    VC_VoiceSetFrequency,
    VC_VoiceGetFrequency,
    VC_VoiceSetPanning,
    VC_VoiceGetPanning,
    VC_VoicePlay,
    VC_VoiceStop,
    VC_VoiceStopped,
    VC_VoiceGetPosition,
    VC_VoiceRealVolume
};
