/* avcodec API use example.
 *
 * Note that this library only handles codecs (mpeg, mpeg4, etc...),
 * not file formats (avi, vob, etc...). See library 'libav' for the
 * format handling 
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <pthread.h>

#include "avcodec.h"
#include "id3.h"
#include "info.h"

#include "../../musicd.h"

input_plugin *get_iplugin_info (void);
static void init(void);
static int is_our_file(char *filename);
static void play(char *filename);
static void stop(void);
static void* play_loop(void*);
static int get_time(void);
static metainfo* get_song_info(char *filename);

input_plugin meta_input_ip =
{
    42,
    NULL,
    NULL,
    "MPEG-1 layer 3 input plugin",
    init,
    is_our_file,
    play,
    stop,
    NULL,
    NULL,
    get_time,
    NULL,
    get_song_info,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL
};

static FILE *file = NULL;
static pthread_t decode_thread;
static int going = 0;
static int audio = 0;

#define INBUF_SIZE 4096


input_plugin *get_iplugin_info()
{
    return(&meta_input_ip);
}

static void init()
{
    avcodec_init();
    audio = 0;
    /* register all the codecs (you can also register only the codec
       you wish to have smaller code */
    register_avcodec(&mp2_decoder);   
    register_avcodec(&mp3_decoder);
    return;
}

static int is_our_file(char *filename)
{
    if(strstr(filename, ".mp3") != NULL)
	return(1);
    if(strstr(filename, ".mp2") != NULL)
	return(1);
    return(0);
}


/*
 * Audio encoding example 
 */
void* play_loop(void* none)
{
    AVCodec *codec;
    AVCodecContext *c= NULL;
    int out_size, size, len;
    UINT8 *outbuf;
    UINT8 inbuf[INBUF_SIZE], *inbuf_ptr;
    int position;
    int to_write;
    
    if(none){}

    /* find the mpeg audio decoder */
    codec = avcodec_find_decoder(CODEC_ID_MP2);
    if (!codec)
    {
        fprintf(stderr, "codec not found\n");
        exit(1);
    }

    c= avcodec_alloc_context();

    /* open it */
    if (avcodec_open(c, codec) < 0)
    {
        fprintf(stderr, "could not open codec\n");
        exit(1);
    }
    
    outbuf = malloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);

    /* decode until eof */
    inbuf_ptr = inbuf;
    while(going)
    {
        size = fread(inbuf, 1, INBUF_SIZE, file);
        if (size == 0)
            break;

        inbuf_ptr = inbuf;
        while (size > 0)
	{
            len = avcodec_decode_audio(c, (short *)outbuf, &out_size, 
                                       inbuf_ptr, size);
            if (len < 0)
	    {
                fprintf(stderr, "Error while decoding\n");
                exit(1);
            }
            if (out_size > 0)
	    {
		position = 0;
		while(out_size)
		{
		    to_write = meta_input_ip.buffer_free();
		    if( to_write > out_size) to_write = out_size;
		    if( to_write == 0 )
		    {   
			usleep(10000);
		    }
		    else
		    {
			meta_input_ip.write_audio(&outbuf[position], 
							  to_write);
			position += to_write;
			out_size -= to_write;
		    }
		}
            }
            size -= len;
            inbuf_ptr += len;
        }
    }

    going = 0;
    free(outbuf);

    avcodec_close(c);
    free(c);
    return(0);
}


static void play(char *filename)
{
    static struct format audio_format;
    int channels = 1;

    memset(&audio_format, 0, sizeof(struct format));
    
    if(!going)
    {
	if(read_format(&audio_format, filename) == -1)
	    return;

	if(audio_format.channel < 3)
	    channels = 2;

	audio = 1;
	meta_input_ip.open_audio(FMT_S16_LE, audio_format.rate, channels);

	file = fopen(filename,"r");
	if(file != NULL)
	{
	    going = 1;
	    pthread_create(&decode_thread, NULL, play_loop, NULL);
	}
    }
}

static void stop(void)
{   
    if(audio)
    {
	going = 0;
	audio = 0;
	pthread_join(decode_thread, NULL);
	if(file)
	    fclose(file);
	meta_input_ip.close_audio();
    }
}

static int get_time(void)
{   
    if(going)
    {
	return(10);
    }
    return(-1);
}


static metainfo* get_song_info(char *filename)
{
    static struct format format;
    static struct id3tag tag;
    static struct metainfo info;
    int channels;
    
    memset(&format, 0, sizeof(struct format));
    memset(&tag, 0, sizeof(struct id3tag));
    memset(&info, 0, sizeof(struct metainfo));

    if(read_format(&format, filename) == -1)
	return(0);

    if(read_tag(&tag, filename) > 0)
    {
	if(tag.artist[0])info.artist = tag.artist;
	if(tag.title[0])info.song = tag.title;
	info.genre = genres[(int)tag.genre];
	info.year = atoi(tag.year);
    }

    if(format.channel < 3)
	channels = 2;

    info.channels = channels;
    info.rate = format.rate;
    info.length = 0;
    info.bitrate = format.bitrate;
    info.format = "MPEG-1 Layer 3";

    return(&info);
}
