/* ogg.c -- Ogg Vorbis plugin for musicdaemon player
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <pthread.h>
#include <stdio.h>
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "../../musicd.h"
#include "config.h"

input_plugin *get_iplugin_info (void);
static void init(void);
static int is_our_file(char *filename);
static void play(char *filename);
static void stop(void);
static void* play_loop(void*);
static int get_time(void);
static void seek(int time);

input_plugin meta_input_ip =
{
    42,
    NULL,
    NULL,
    "Ogg input plugin",
    init,
    is_our_file,
    play,
    stop,
    NULL,
    seek,
    get_time,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL
};

static FILE *file = NULL;
static pthread_t decode_thread;
static int going = 0;
static int audio;
static OggVorbis_File vf;
static pthread_mutex_t vf_mutex = PTHREAD_MUTEX_INITIALIZER;


input_plugin *get_iplugin_info()
{
    return(&meta_input_ip);
}

static void init()
{
    audio = 0;
    return;
}

static int is_our_file(char *filename)
{
    if(strstr(filename, ".ogg") != NULL)
	return(1);
    return(0);
}

static void* play_loop(void* none)
{
    char buf[4096];
    int written;
    int read;
    int current_section;
    int to_write;
    
    if(none){}
        
    while(going)
    {
	written = 0;
	pthread_mutex_lock(&vf_mutex);
#ifdef WORDS_BIGENDIAN
	read = ov_read(&vf,buf,sizeof(buf),1,2,1,&current_section);
#else
	read = ov_read(&vf,buf,sizeof(buf),0,2,1,&current_section);
#endif
	pthread_mutex_unlock(&vf_mutex);
	if(read == 0) break;
	while(written < read)
	{
	    to_write = meta_input_ip.buffer_free();
	    if( to_write == 0 )
	    {
		usleep(10000);
		continue;
	    }
	    if( to_write > (read-written))
	    {
		meta_input_ip.write_audio(&buf[written], read-written);
		break;
	    }
	    meta_input_ip.write_audio(&buf[written], to_write);
	    written += to_write;
	}
    }
    going = 0;
    ov_clear(&vf);
    return(0);
}

static void play(char *filename)
{
    vorbis_info *info;
    if(!going)
    {
	going = 1;
	audio = 1;
	file = fopen(filename,"r");
	if(file != NULL)
	{
	    if(ov_open(file, &vf, NULL, 0) < 0)
	    {   
		perror("ov_open");
		going = 0;
		return;
	    }
	    info = ov_info(&vf, -1);
	    if(!info)
	    {
		perror("ov_info");
		going = 0;
		return;
	    }
#ifdef WORDS_BIGENDIAN
	    meta_input_ip.open_audio(FMT_S16_BE, info->rate, info->channels);
#else
	    meta_input_ip.open_audio(FMT_S16_LE, info->rate, info->channels);
#endif
	    pthread_create(&decode_thread, NULL, play_loop, NULL);
	}
	else
	    going = 0;
    }
}

static void stop(void)
{
    if(audio)
    {
	audio = 0;
	going = 0;
	pthread_join(decode_thread, NULL);
	meta_input_ip.close_audio();
    }
}

static int get_time(void)
{
    if(going)
    {
	return(10);
    }
    return(-1);
}

static void seek(int time)
{
    pthread_mutex_lock(&vf_mutex);
    ov_time_seek(&vf, time);
    pthread_mutex_unlock(&vf_mutex);
    meta_input_ip.flush(time);
}
