/* list.c -- List handling for musicdaemon player
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdlib.h>
#include "list.h"

void list_init(list *list)
{
    list->start = 0;
}

int list_add(void *data, int position, list *list)
{
    int tmp;
    list_node *point;
    list_node *item;
	
    if(list == 0)
	return(1);
	
    point = list->start;

    item = calloc(1, sizeof(list_node));
    if(item == 0)
	return(-1);
    item->data = data;
    
    if(position == 0 || point == 0)
    {
	item->next = list->start;
	if(list->start)
	    list->start->prev = item;
	list->start = item;
	return(0);
    }
	
    for(tmp = 1; tmp < position; tmp++)
    {
	if(point->next == 0)
	    break;
	point = point->next;
    }

    item->next = point->next;
    item->prev = point;
    if(point->next)
	point->next->prev = item;
    point->next = item;

    return(0);
}

void *list_get(int position, list *list)
{
    list_node *point = get_node(position, list);
    if(point == 0)
	return(0);
	
    return(point->data);
}

list_node *get_node(int position, list *list)
{
    int tmp;
    list_node *point;
	
    if(list == 0)
	return(0);

    point = list->start;
	
    for(tmp = 1; tmp < position; tmp++)
    {
	if(point == 0)
	    return(0);
	point = point->next;
    }
	
    return(point);
}

int list_size(list *list)
{
    int tmp;
    list_node *point;
    if(list == 0 || list->start == 0)
	return(0);
    point = list->start;
    for(tmp = 0; point; tmp++)
	point = point->next;

    return(tmp);
}


int list_delete(int position, list *list)
{
    list_node* point = get_node(position, list);
    if(point == 0)
	return(-1);
    if(point == list->start)
	list->start = point->next;
    if(point->next)
	point->next->prev = point->prev;
    if(point->prev)
	point->prev->next = point->next;
    free(point);
    return(0);
}
