/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 8 -*- */

/* main.c -- Main for musicdaemon player
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/select.h>
#include <string.h>
#include <signal.h>
#include <dlfcn.h>
#include <time.h>


#include "musicd.h"
#include "config.h"
#include "network.h"
#include "playlist.h"
#include "main.h"
#include "plugin.h"
#include "list.h"
#include "init.h"
#include "signals.h"

extern struct core *sig_core;

int main(int argc, char *argv[])
{
    core core;
    struct timeval timeout;
    command commands[] = {
	{"ADD", cmd_add},
	{"ADD_AT", cmd_add_at},
	{"CLEAR", cmd_clear},
	{"JUMP", cmd_jump},
	{"LOAD", cmd_load},
	{"MODULELOAD", cmd_moduleload},
	{"MODULESCANDIR", cmd_modulescandir},
	{"MOVE", cmd_move},
	{"MUTE", cmd_mute},
	{"NEXT", cmd_next},
	{"PAUSE", cmd_pause},
	{"PLAY", cmd_play},
	{"PREV", cmd_prev},
	{"REMOVE", cmd_remove},
	{"SAVE", cmd_save},
	{"SEARCH", cmd_search},
	{"SEEK", cmd_seek},
	{"SET", cmd_set},
	{"SHOWLIST", cmd_show},
	{"SHOWPLUGINS", cmd_showplugins},
	{"SHUTDOWN", cmd_shutdown},
	{"STATS", cmd_stats},
	{"STOP", cmd_stop},
	{"VOLUME", cmd_volume},
	{NULL, NULL}
    };

    signal(SIGPIPE, SIG_IGN);
    signal(SIGINT, signal_interrupt);
    signal(SIGTERM, signal_terminate);
    sig_core = &core;

    memset(&core, 0, sizeof(struct core));
    
    core.commands = commands;

    if(init(&core, argc, argv) == -1)
	set_flag(core.status, STATUS_QUIT);
	    
    if(core.output.start && !core.active_output)
	core.active_output = (struct output_plugin *) core.output.start->data;

    while(1)
    {
	if(has_flag(core.status, STATUS_QUIT))
	    break;
	timeout.tv_sec = 1;
	timeout.tv_usec = 0;
	wait_input(&core, &timeout);
	if(has_flag(core.status, STATUS_PLAYING))
	{
	    if((*core.active_input->get_time)() == -1)
	    {
		if(has_flag(core.status, STATUS_PLAYLIST))
		    cmd_next(&core, NULL, NULL);
		else
		    cmd_stop(&core, NULL, NULL);
	    }
	}
    }

    network_close(&core);
    return(0);
}
