#ifndef MAIN_H
#define MAIN_H

#define MAX_MSG_SIZE 512

#define STATUS_PLAYING       0x00000001
#define STATUS_PLAYLIST      0x00000002
#define STATUS_RANDOM        0x00000004
#define STATUS_LOOP          0x00000008
#define STATUS_PAUSED        0x00000010
#define STATUS_QUIT          0x80000000

#define has_flag(status, flag) ((status) & (flag))
#define set_flag(status, flag) status = (status) | (flag)
#define unset_flag(status, flag) status = (status) & (~(flag))
#define swap_flag(status, flag) status = (status) ^ (flag)

struct core;

#include "playlist.h"
#include "network.h"
#include "commands.h"
#include "plugin.h"
#include "list.h"

typedef struct stats
{
    int uptime;
    unsigned long int songs;
    unsigned long int playtime;
    int song_start;
} stats;

typedef struct core
{
    int listen_port;	/* Portnumber and socket for network connections */
    int network;

    char *unixfile;	/* Filename and socket for Unix socket */
    int unixdomain;

    char *logfile;
    char *configfile;
    int logfd;
    
    int status;
    
    int seed;
    int rands;
    
    struct stats stats;

    struct input_plugin *active_input;
    struct output_plugin *active_output;
    struct playlist *active_playlist;

    list clients;	/* struct client */
    list playlist;	/* struct playlist */
    list input;		/* struct input_plugin */
    list output;	/* struct output_plugin */
    list modpaths;	/* char * */
    struct command *commands;
} core;

int main(int argc, char *argv[]);

#endif
