#ifndef MUSICD_H
#define MUSICD_H

/* Information about audiofile, if information is not available value should
 * be 0 or NULL.
 */
typedef struct metainfo
{   
    /* Always present metainformation */
    char *filename; /* Filled by musicd */
    char *path;     /* Filled by musicd */
    int playlist;   /* Filled by musicd */
    int channels;
    int rate;	    /* Sample rate in Hz */
    
    /* Optional metainformation */
    int length;	    /* Estimated length of the song in seconds */
    char *artist;
    char *song;
    int bitrate;
    char *format;
    char *genre;
    int year;
    int track;
} metainfo;


typedef enum
{       
    FMT_U8,
    FMT_S8,
    FMT_U16_LE,
    FMT_U16_BE,
    FMT_S16_LE,
    FMT_S16_BE,
} audio_format;


typedef struct output_plugin
{
    int plugin_id;	/* If 42 this is version 1 plugin */
    void *handle;	/* Filled by musicd */
    char *filename;	/* Filled by musicd */
    char *description;	/* Description of the plugin */
    
    /* Functions for player-plugin interaction */
    
    void (*init) (void);	/* Initialization, called on load */
    void (*get_volume) (int *l, int *r); /* Get current volume */
    void (*set_volume) (int l, int r);	 /* Set current volume */
    int (*open_audio) (audio_format fmt, int rate, int nch);
    /* Open audio for given format */
    
    void (*write_audio) (void *ptr, int length); /* Write given data */
    void (*close_audio) (void);	  /* Close audio and clean up */
    void (*flush) (int time);
    void (*pause) (short paused); /* Pause(>=1) and unpause(0) */
    int (*buffer_free) (void);	  /* Return the free space in buffer */
    int (*buffer_playing) (void);
    int (*output_time) (void);
    int (*written_time) (void);
} output_plugin;


typedef struct input_plugin
{
    int plugin_id;	/* If 42 this is version 1 plugin */
    void *handle;	/* Filled by musicd */
    char *filename;	/* Filled by musicd */
    char *description;	/* Description of the plugin */

    /* Functions for player-plugin interaction */
    /* Filled by plugin, called by musicd */
    void (*init)(void);		 	/* Initialization, called on load */
    int (*is_our_file)(char *file); 	/* Check if file can be played by us */
    void (*play_file)(char *filename); 	/* Play given file */
    void (*stop)(void);			/* Stop playing and go passive */
    void (*pause)(short paused);	/* Pause(>=1) and unpause(0) */
    void (*seek)(int time); /* Seek to given position (time in seconds) */
    int	(*get_time) (void); /* Current playing time */
    void (*cleanup) (void); /* Called on exit and plugin deactivation */
    metainfo *(*get_song_info) (char *filename);
    /* Return metainformation. Metainfo should be static data and remain
     * untouched until next call to get_song_info */
    
    /* Filled by musicd called by plugin */
    void (*write_audio) (void *ptr, int length); /* Write given data */
    int (*buffer_free) (void);    /* Return the free space in buffer */
    int (*open_audio) (audio_format fmt, int rate, int nch);
    /* Open audio for given format */
    void (*close_audio) (void);   /* Close audio and clean up */
    void (*flush) (int time);
} input_plugin;

#endif
