/* parse.c -- Command parsing code for musicdaemon player
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <sys/select.h>
#include <stdio.h>
#include <string.h>

#include "network.h"
#include "parse.h"
#include "config.h"
#include "log.h"

int parse_client(core *core, client *user)
{
    char line[513];
    int point;
    int newline;
    
    for(newline = 0; ; newline = 0)
    {
	for(point = 0; point < user->bufferfill; point++)
	{
	    line[point] = user->buffer[point];
	    if(line[point] == '\n')
	    {
		line[point] = 0;
		
		if(line[point-1] == '\r')
		    line[point-1] = 0;
		
		point++;
		newline = 1;
		break;
	    }
	    if(point == 511)
	    {
		point++;
		line[point] = 0;
		newline = 1;
		break;
	    }
	}
	
	if(!newline) break;
	
	if(user->bufferfill != point)
	    memcpy(user->buffer, &user->buffer[point], 
		   sizeof(user->buffer) - point);
	
	user->bufferfill -= point;
    
	parse_line(core, user, line);
	
    }
    
    return(0);
}


int parse_line(core *core, client *user, char *line)
{
    char *params;
    int cmd;

    if(strlen(line) == 0) return(0);

    params = strchr(line, ' ');
    
    if(params)
    {
	*params = 0;
	params++;
    }
    
    for(cmd = 0; core->commands[cmd].command; cmd++)
    {
	if(strcasecmp(line, core->commands[cmd].command) == 0)
	{
	    (*core->commands[cmd].func)(core, user, params);
	    return(0);
	}
    }
    
    log_printf(core, "Unknow command: %s", line);
    
    return(0);
}
