#include <time.h>
#include <stdlib.h>
#include <string.h>

#include "main.h"
#include "musicd.h"
#include "playlist.h"
#include "log.h"

int broadcast_songinfo(core *core, song *song)
{
    broad_printf(core,"SONG\n");
    if(song->info.filename)
	broad_printf(core,"FILENAME: %s\n", song->info.filename);

    if(song->info.path)
	broad_printf(core,"PATH: %s\n", song->info.path);

    broad_printf(core,"PLAYLIST: %d\n", song->info.playlist);

    if(song->info.channels)
	broad_printf(core,"CHANNELS: %d\n", song->info.channels);
    if(song->info.rate)
	broad_printf(core,"RATE: %d\n",song->info.rate);
    
    if(song->info.length)
	broad_printf(core,"LENGTH: %d\n",song->info.length);
    
    if(song->info.artist)
	broad_printf(core,"ARTIST: %s\n",song->info.artist);
    
    if(song->info.song)
	broad_printf(core,"SONG: %s\n",song->info.song);
    
    if(song->info.bitrate)
	broad_printf(core,"BITRATE: %d\n",song->info.bitrate);
    
    if(song->info.format)
	broad_printf(core,"FORMAT: %s\n",song->info.format);
    
    if(song->info.genre)
	broad_printf(core,"GENRE: %s\n",song->info.genre);
    
    if(song->info.year)
	broad_printf(core,"YEAR: %d\n",song->info.year);

    if(song->info.track)
	broad_printf(core,"TRACK: %d\n",song->info.track);

    broadcast(core,"END SONG\n");
    return(0);
}

int play_song(core *core, song *song)
{
    list_node *input;
    input_plugin *plugin;

    list_for_each(input, &core->input)
    {   
	plugin = input->data;
	if((plugin->is_our_file)(song->file))
	{
	    core->active_input = plugin;
	    plugin->write_audio = core->active_output->write_audio;
	    plugin->buffer_free = core->active_output->buffer_free;
	    plugin->close_audio = core->active_output->close_audio;
	    plugin->open_audio = core->active_output->open_audio;
	    plugin->flush = core->active_output->flush;
	    
	    log_printf(core,"Playing: %s ...",song->file);
	    (plugin->play_file)(song->file);
	    set_flag(core->status, STATUS_PLAYING);
	    core->stats.song_start = time(0);
	    core->stats.songs++;
	    broadcast_songinfo(core, song);
	    broadcast(core, "PLAY\n");
	    return(0);
	}
    } 
    return(1);
}

int update_songinfo(core *core, song *song, int pos)
{
    list_node *input;
    input_plugin *plugin;
    metainfo *info;
    char *path;
    char *filename;
    int free_info = 0;

    list_for_each(input, &core->input)
    {   
	plugin = input->data;
	if((plugin->is_our_file)(song->file))
	{
	    
	    if(plugin->get_song_info)
	    {
		info = (plugin->get_song_info)(song->file);
	    }
	    else
	    {
		info = calloc(1, sizeof(metainfo));
		free_info = 1;
	    }
	    info->playlist = pos;
	    path = strdup(song->file);
	    filename = strrchr(path, '/');
	    if(filename)
	    {
		*filename = 0;
		info->filename = strdup(filename+1);
	    }
	    info->path = strdup(path);
	    free(path);
	    copy_metainfo(info, &song->info);
	    free(info->path);
	    info->path = 0;
	    free(info->filename);
	    info->filename = 0;
	    if(free_info)
		free(info);
	    break;
	}
    }
    return(0);
}


int copy_metainfo(metainfo *from, metainfo *to)
{
    free_metainfo(to);
    if(from->filename) to->filename = strdup(from->filename);
    if(from->path) to->path = strdup(from->path);
    to->playlist = from->playlist;
    to->channels = from->channels;
    to->rate = from->rate;
    to->length = from->length;
    if(from->artist) to->artist = strdup(from->artist);
    if(from->song) to->song = strdup(from->song);
    to->bitrate = from->bitrate;
    if(from->format) to->format = strdup(from->format);
    if(from->genre) to->genre = strdup(from->genre);
    to->year = from->year;
    to->track = from->track;
    return(0);
}

int free_metainfo(metainfo *info)
{
    free(info->filename);
    free(info->path);
    free(info->artist);
    free(info->song);
    free(info->format);
    free(info->genre);
    return(0);
}

