/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 8 -*- */

/* plugin.c -- Plugin code for musicdaemon player
 *
 * Copyright (C) 2002-2003 Petri Lahtinen <pjl@timefade.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <glob.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <unistd.h>

#include "plugin.h"
#include "musicd.h"
#include "log.h"


int plugin_scan_dir(core *core, char *directory)
{
    glob_t glob_buf;
    unsigned int point;
    char *pattern = calloc(1, strlen(directory) + 6);

    if(!pattern)
	return(-1);

    if(directory[strlen(directory)-1] != '/')
	sprintf(pattern, "%s/*.so", directory);
    else
	sprintf(pattern, "%s*.so", directory);

    if(glob(pattern, GLOB_NOESCAPE, NULL, &glob_buf) != 0)
	return(-1);

    for(point = 0; point < glob_buf.gl_pathc; point++)
    {
	plugin_load(core, glob_buf.gl_pathv[point]);
    }

    free(pattern);
    globfree(&glob_buf);
    return(0);
}

int plugin_load(core *core, char *path)
{
    void *handle;
    list_node *plugin;
    input_plugin *(*get_iplugin_info)(void);
    output_plugin *(*get_oplugin_info)(void);
    input_plugin *input;
    output_plugin *output;

    list_for_each(plugin, &core->input)
    {
	if(strcmp(((input_plugin*)plugin->data)->filename, path) == 0)
	    return(0);
    }

    list_for_each(plugin, &core->output)
    {
	if(strcmp(((output_plugin*)plugin->data)->filename, path) == 0)
	    return(0);
    }
    
    handle = dlopen(path, RTLD_NOW);
    if(!handle)
    {
	perror(dlerror());
	return(-1);
    }

    get_iplugin_info = dlsym(handle, "get_iplugin_info");
    if(get_iplugin_info)
    {
	input = (*get_iplugin_info)();
	if(input->plugin_id == 42)
	{
	    list_add(input, list_size(&core->input) + 1, &core->input);
	    (*input->init)();
	    input->handle = handle;
	    input->filename = strdup(path);
	    log_printf(core, "Loaded: %s", input->description);
	}
	return(0);
    }

    get_oplugin_info = dlsym(handle, "get_oplugin_info");
    if(get_oplugin_info)
    {
	output = (*get_oplugin_info)();
	if(output->plugin_id == 42)
	{
	    list_add(output, list_size(&core->output) + 2, &core->output);
	    (*output->init)();
	    output->handle = handle;
	    output->filename = strdup(path);
	    log_printf(core, "Loaded: %s", output->description);
	}
	return(0);
    }

    log_printf(core, "%s is not a plugin", path);
    return(1);
}
